@@ -230,13 +230,29 @@ public class RdbChecksumStore implements ChecksumStore {
 
     @Override
     public int removeFileChecksumsInFolders(List<FolderID> folderIDs) throws OXException {
+        int deleted = 0;
         Connection connection = databaseService.getWritable(contextID);
         try {
-            return deleteFileChecksumsInFolders(connection, contextID, folderIDs.toArray(new FolderID[folderIDs.size()]));
+            for (int i = 0; i < folderIDs.size(); i += DELETE_CHUNK_SIZE) {
+                /*
+                 * prepare chunk
+                 */
+                int length = Math.min(folderIDs.size(), i + DELETE_CHUNK_SIZE) - i;
+                List<FolderID> chunk = folderIDs.subList(i, i + length);
+                /*
+                 * delete chunk
+                 */
+                deleted += deleteFileChecksumsInFolders(connection, contextID, chunk.toArray(new FolderID[chunk.size()]));
+            }
+            return deleted;
         } catch (SQLException e) {
             throw DriveExceptionCodes.DB_ERROR.create(e, e.getMessage());
         } finally {
-            databaseService.backWritable(contextID, connection);
+            if (0 < deleted) {
+                databaseService.backWritable(contextID, connection);
+            } else {
+                databaseService.backWritableAfterReading(contextID, connection);
+            }
         }
     }
 
@@ -275,7 +291,11 @@ public class RdbChecksumStore implements ChecksumStore {
         } catch (SQLException e) {
             throw DriveExceptionCodes.DB_ERROR.create(e, e.getMessage());
         } finally {
-            databaseService.backWritable(contextID, connection);
+            if (0 < deleted) {
+                databaseService.backWritable(contextID, connection);
+            } else {
+                databaseService.backWritableAfterReading(contextID, connection);
+            }
         }
         return deleted;
     }
@@ -336,13 +356,26 @@ public class RdbChecksumStore implements ChecksumStore {
     public List<DirectoryChecksum> insertDirectoryChecksums(List<DirectoryChecksum> directoryChecksums) throws OXException {
         Connection connection = databaseService.getWritable(contextID);
         try {
-            for (DirectoryChecksum directoryChecksum : directoryChecksums) {
-                if (null != directoryChecksum.getUuid()) {
-                    throw new IllegalArgumentException("New directory checksums must not contain an UUID");
+            for (int i = 0; i < directoryChecksums.size(); i += INSERT_CHUNK_SIZE) {
+                /*
+                 * prepare chunk
+                 */
+                int length = Math.min(directoryChecksums.size(), i + INSERT_CHUNK_SIZE) - i;
+                DirectoryChecksum[] checksums = new DirectoryChecksum[length];
+                for (int j = 0; j < length; j++) {
+                    DirectoryChecksum checksum = directoryChecksums.get(i + j);
+                    if (null != checksum.getUuid()) {
+                        throw new IllegalArgumentException("New directory checksums must not contain an UUID");
+                    }
+                    checksum.setUuid(newUid());
+                    checksums[j] = checksum;
                 }
-                directoryChecksum.setUuid(newUid());
-                if (0 == insertDirectoryChecksum(connection, contextID, directoryChecksum)) {
-                    throw DriveExceptionCodes.DB_ERROR.create("File checksum not added: " + directoryChecksum);
+                /*
+                 * insert chunk
+                 */
+                int inserted = insertDirectoryChecksums(connection, contextID, checksums);
+                if (checksums.length != inserted) {
+                    throw DriveExceptionCodes.DB_ERROR.create(String.valueOf(checksums.length - inserted) + " directory checksums not inserted");
                 }
             }
         } catch (SQLException e) {
@@ -390,11 +423,11 @@ public class RdbChecksumStore implements ChecksumStore {
 
     @Override
     public boolean removeDirectoryChecksum(FolderID folderID) throws OXException {
-        return 0 < removeDirectoryChecksums(Collections.singletonList(folderID));
+        return 0 < removeAllDirectoryChecksums(Collections.singletonList(folderID));
     }
 
     @Override
-    public int removeDirectoryChecksums(List<FolderID> folderIDs) throws OXException {
+    public int removeAllDirectoryChecksums(List<FolderID> folderIDs) throws OXException {
         Connection connection = databaseService.getWritable(contextID);
         try {
             return deleteDirectoryChecksums(connection, contextID, folderIDs.toArray(new FolderID[folderIDs.size()]));
@@ -425,9 +458,9 @@ public class RdbChecksumStore implements ChecksumStore {
 
     @Override
     public int touchDirectoryChecksums(List<DirectoryChecksum> directoryChecksums) throws OXException {
+        int touched = 0;
         Connection connection = databaseService.getWritable(contextID);
         try {
-            int touched = 0;
             for (int i = 0; i < directoryChecksums.size(); i += INSERT_CHUNK_SIZE) {
                 /*
                  * prepare chunk
@@ -446,7 +479,77 @@ public class RdbChecksumStore implements ChecksumStore {
                  */
                 touched += touchDirectoryChecksums(connection, contextID, uuids);
             }
-            return touched;
+        } catch (SQLException e) {
+            throw DriveExceptionCodes.DB_ERROR.create(e, e.getMessage());
+        } finally {
+            if (0 < touched) {
+                databaseService.backWritable(contextID, connection);
+            } else {
+                databaseService.backWritableAfterReading(contextID, connection);
+            }
+        }
+        return touched;
+    }
+
+    @Override
+    public List<DirectoryChecksum> getUnusedDirectoryChecksums(long unusedSince) throws OXException {
+        Connection connection = databaseService.getReadOnly(contextID);
+        try {
+            return selectUnusedDirectoryChecksums(connection, contextID, unusedSince);
+        } catch (SQLException e) {
+            throw DriveExceptionCodes.DB_ERROR.create(e, e.getMessage());
+        } finally {
+            databaseService.backReadOnly(contextID, connection);
+        }
+    }
+
+    @Override
+    public List<DirectoryChecksum> getDirectoryChecksums(List<FolderID> folderIDs) throws OXException {
+        if (null == folderIDs) {
+            return Collections.emptyList();
+        }
+        List<DirectoryChecksum> checksums = new ArrayList<DirectoryChecksum>();
+        Connection connection = databaseService.getReadOnly(contextID);
+        try {
+            /*
+             * select chunk-wise
+             */
+            for (int i = 0; i < folderIDs.size(); i += SELECT_WHERE_IN_CHUNK_SIZE) {
+                int length = Math.min(folderIDs.size(), i + SELECT_WHERE_IN_CHUNK_SIZE) - i;
+                checksums.addAll(selectDirectoryChecksums(connection, contextID, folderIDs.subList(i, i + length)));
+            }
+            return checksums;
+        } catch (SQLException e) {
+            throw DriveExceptionCodes.DB_ERROR.create(e, e.getMessage());
+        } finally {
+            databaseService.backReadOnly(contextID, connection);
+        }
+    }
+
+    @Override
+    public int removeDirectoryChecksums(List<DirectoryChecksum> directoryChecksums) throws OXException {
+        Connection connection = databaseService.getWritable(contextID);
+        try {
+            int deleted = 0;
+            for (int i = 0; i < directoryChecksums.size(); i += DELETE_CHUNK_SIZE) {
+                /*
+                 * prepare chunk
+                 */
+                int length = Math.min(directoryChecksums.size(), i + DELETE_CHUNK_SIZE) - i;
+                String[] uuids = new String[length];
+                for (int j = 0; j < length; j++) {
+                    String uuid = directoryChecksums.get(i + j).getUuid();
+                    if (null == uuid) {
+                        throw new IllegalArgumentException("Removing directory checksums requires an existing UUID");
+                    }
+                    uuids[j] = uuid;
+                }
+                /*
+                 * delete chunk
+                 */
+                deleted += deleteDirectoryChecksums(connection, contextID, uuids);
+            }
+            return deleted;
         } catch (SQLException e) {
             throw DriveExceptionCodes.DB_ERROR.create(e, e.getMessage());
         } finally {
@@ -662,19 +765,23 @@ public class RdbChecksumStore implements ChecksumStore {
         return fileChecksums;
     }
 
-    private static int insertDirectoryChecksum(Connection connection, int cid, DirectoryChecksum directoryChecksum) throws SQLException, OXException {
+    private static int insertDirectoryChecksums(Connection connection, int cid, DirectoryChecksum[] directoryChecksums) throws SQLException, OXException {
+        long currentTimeMillis = System.currentTimeMillis();
         PreparedStatement stmt = null;
         try {
-            stmt = connection.prepareStatement(SQL.INSERT_DIRECTORY_CHECKSUM_STMT);
-            stmt.setString(1, directoryChecksum.getUuid());
-            stmt.setInt(2, cid);
-            stmt.setInt(3, directoryChecksum.getUserID());
-            stmt.setInt(4, directoryChecksum.getView());
-            stmt.setString(5, escapeFolder(directoryChecksum.getFolderID()));
-            stmt.setLong(6, directoryChecksum.getSequenceNumber());
-            stmt.setString(7, directoryChecksum.getETag());
-            stmt.setString(8, directoryChecksum.getChecksum());
-            stmt.setLong(9, System.currentTimeMillis());
+            stmt = connection.prepareStatement(SQL.INSERT_DIRECTORY_CHECKSUMS_STMT(directoryChecksums.length));
+            int parameterIndex = 1;
+            for (DirectoryChecksum directoryChecksum : directoryChecksums) {
+                stmt.setString(parameterIndex++, directoryChecksum.getUuid());
+                stmt.setInt(parameterIndex++, cid);
+                stmt.setInt(parameterIndex++, directoryChecksum.getUserID());
+                stmt.setInt(parameterIndex++, directoryChecksum.getView());
+                stmt.setString(parameterIndex++, escapeFolder(directoryChecksum.getFolderID()));
+                stmt.setLong(parameterIndex++, directoryChecksum.getSequenceNumber());
+                stmt.setString(parameterIndex++, directoryChecksum.getETag());
+                stmt.setString(parameterIndex++, directoryChecksum.getChecksum());
+                stmt.setLong(parameterIndex++, currentTimeMillis);
+            }
             return SQL.logExecuteUpdate(stmt);
         } finally {
             DBUtils.closeSQLStuff(stmt);
@@ -715,7 +822,7 @@ public class RdbChecksumStore implements ChecksumStore {
     private static int deleteDirectoryChecksums(Connection connection, int cid, FolderID[] folderIDs) throws SQLException, OXException {
         PreparedStatement stmt = null;
         try {
-            stmt = connection.prepareStatement(SQL.DELETE_DIRECTORY_CHECKSUMS_STMT(folderIDs.length));
+            stmt = connection.prepareStatement(SQL.DELETE_DIRECTORY_CHECKSUMS_FOR_FOLDER_STMT(folderIDs.length));
             stmt.setInt(1, cid);
             for (int i = 0; i < folderIDs.length; i++) {
                 stmt.setString(i + 2, reverse(escapeFolder(folderIDs[i])));
@@ -771,5 +878,72 @@ public class RdbChecksumStore implements ChecksumStore {
         }
     }
 
+    private static List<DirectoryChecksum> selectUnusedDirectoryChecksums(Connection connection, int cid, long unusedSince) throws SQLException, OXException {
+        List<DirectoryChecksum> directoryChecksums = new ArrayList<DirectoryChecksum>();
+        PreparedStatement stmt = null;
+        try {
+            stmt = connection.prepareStatement(SQL.SELECT_UNUSED_DIRECTORY_CHECKSUMS_STMT);
+            stmt.setInt(1, cid);
+            stmt.setLong(2, unusedSince);
+            ResultSet resultSet = SQL.logExecuteQuery(stmt);
+            while (resultSet.next()) {
+                DirectoryChecksum directoryChecksum = new DirectoryChecksum();
+                directoryChecksum.setUuid(resultSet.getString(1));
+                directoryChecksum.setFolderID(unescapeFolder(resultSet.getString(2)));
+                directoryChecksum.setSequenceNumber(resultSet.getLong(3));
+                directoryChecksum.setETag(resultSet.getString(4));
+                directoryChecksum.setUserID(resultSet.getInt(5));
+                directoryChecksum.setView(resultSet.getInt(6));
+                directoryChecksum.setChecksum(resultSet.getString(7));
+                directoryChecksums.add(directoryChecksum);
+            }
+        } finally {
+            DBUtils.closeSQLStuff(stmt);
+        }
+        return directoryChecksums;
+    }
+
+    private static List<DirectoryChecksum> selectDirectoryChecksums(Connection connection, int cid, List<FolderID> folderIDs) throws SQLException, OXException {
+        List<DirectoryChecksum> directoryChecksums = new ArrayList<DirectoryChecksum>();
+        PreparedStatement stmt = null;
+        try {
+            stmt = connection.prepareStatement(SQL.SELECT_ALL_DIRECTORY_CHECKSUMS_STMT(folderIDs.size()));
+            int parameterIndex = 1;
+            stmt.setInt(parameterIndex++, cid);
+            for (FolderID folderID : folderIDs) {
+                stmt.setString(parameterIndex++, reverse(escapeFolder(folderID)));
+            }
+            ResultSet resultSet = SQL.logExecuteQuery(stmt);
+            while (resultSet.next()) {
+                DirectoryChecksum directoryChecksum = new DirectoryChecksum();
+                directoryChecksum.setUuid(resultSet.getString(1));
+                directoryChecksum.setFolderID(unescapeFolder(resultSet.getString(2)));
+                directoryChecksum.setSequenceNumber(resultSet.getLong(3));
+                directoryChecksum.setETag(resultSet.getString(4));
+                directoryChecksum.setUserID(resultSet.getInt(5));
+                directoryChecksum.setView(resultSet.getInt(6));
+                directoryChecksum.setChecksum(resultSet.getString(7));
+                directoryChecksums.add(directoryChecksum);
+            }
+        } finally {
+            DBUtils.closeSQLStuff(stmt);
+        }
+        return directoryChecksums;
+    }
+
+    private static int deleteDirectoryChecksums(Connection connection, int cid, String[] uuids) throws SQLException {
+        PreparedStatement stmt = null;
+        try {
+            stmt = connection.prepareStatement(SQL.DELETE_DIRECTORY_CHECKSUMS_STMT(uuids.length));
+            stmt.setInt(1, cid);
+            for (int i = 0; i < uuids.length; i++) {
+                stmt.setBytes(i + 2, SQL.getBytes(uuids[i]));
+            }
+            return SQL.logExecuteUpdate(stmt);
+        } finally {
+            DBUtils.closeSQLStuff(stmt);
+        }
+    }
+
 }
 
